"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const html_minifier_1 = require("html-minifier");
const jsdom_1 = require("jsdom");
const luxon_1 = require("luxon");
const url_1 = require("url");
const resource_1 = require("../../src/models/resource");
const plugins_1 = require("../plugins");
const css_1 = require("../utils/css");
const element_1 = require("../utils/element");
const file_1 = require("../utils/file");
const twitter_1 = require("../utils/twitter");
class VJsdom {
    constructor(html, options = {}) {
        this.doctype = '<!DOCTYPE html>';
        const virtualConsole = new jsdom_1.VirtualConsole();
        this.dom = new jsdom_1.JSDOM(html, Object.assign({}, options, { virtualConsole }));
        const meta = this.metaElement;
        if (meta) {
            this.metadata = VJsdom.parseMetadata(meta);
        }
        virtualConsole.sendTo(console, { omitJSDOMErrors: true });
    }
    static parseMetadata(meta) {
        const parsed = JSON.parse(meta.content);
        return Object.assign({}, parsed, { _createdAt: luxon_1.DateTime.fromISO(parsed._createdAt).toUTC() });
    }
    get document() {
        return this.dom.window.document;
    }
    get location() {
        return this.dom.window.location;
    }
    get metaElement() {
        return this.document.querySelector('meta[name=vanilla-clipper]');
    }
    generate() {
        const html = `${this.doctype}\n${this.document.documentElement.outerHTML}`;
        const url = this.location.href;
        const minified = (() => {
            try {
                return html_minifier_1.minify(html, { minifyJS: true, minifyCSS: true });
            }
            catch (error) {
                return html;
            }
        })();
        return {
            html: minified,
            url,
            createdAt: this.metadata && this.metadata._createdAt,
        };
    }
    finder(...selectors) {
        return [...this.document.querySelectorAll(element_1.selectorsToString(...selectors))];
    }
    execPlugins() {
        plugins_1.jsdomPlugins.exec(this);
    }
    setElementAsRoot(selector) {
        const el = this.document.body.querySelector(selector);
        if (el) {
            this.document.body.innerHTML = el.outerHTML;
        }
    }
    getIframes() {
        return this.finder(element_1.buildVAttrSelector.iframeUuid());
    }
    replaceVideoElement(oldElement, url, isGif) {
        const el = this.document.createElement('video');
        el.dataset.vanillaClipperVideo = isGif ? 'gif' : 'video';
        el.src = url;
        el.style.width = '100%';
        el.style.height = '100%';
        el.style.position = 'relative';
        if (isGif) {
            el.setAttribute('muted', '');
        }
        el.autoplay = isGif;
        el.loop = isGif;
        el.controls = true;
        // oldElement.closest('.PlayableMedia-reactWrapper')!.replaceWith(el)
        oldElement.parentElement.parentElement.parentElement.replaceWith(el);
    }
    async embedTwitterVideo(tweetID) {
        const gifElement = this.document.querySelector('img[src*="/tweet_video_thumb/"]');
        const videoElement = this.document.querySelector('img[src*="/ext_tw_video_thumb/"]');
        if (gifElement) {
            const media = await twitter_1.getVideoURLInTweet(tweetID);
            if (!media)
                return;
            this.replaceVideoElement(gifElement, media.url, true);
        }
        if (videoElement) {
            const media = await twitter_1.getVideoURLInTweet(tweetID);
            if (!media)
                return;
            this.replaceVideoElement(videoElement, media.url, false);
        }
    }
    embedIFrameContents(iframeDataList) {
        return iframeDataList.map(({ uuid, html }) => {
            const iframe = this.finder(element_1.buildVAttrSelector.iframeUuid(uuid))[0];
            delete iframe.dataset.vanillaClipperIframeUuid;
            iframe.srcdoc = html;
            iframe.dataset.vanillaClipperSrc = iframe.src;
            iframe.removeAttribute('src');
            return iframe;
        });
    }
    async processResourcesInAttrs(noStoring = false) {
        const tasks = [];
        const process = (el, attrName, vcPropName) => {
            const originalUrl = el.getAttribute(attrName);
            if (file_1.dataURLPattern.test(originalUrl)) {
                return;
            }
            el.dataset[vcPropName] = originalUrl;
            if (noStoring) {
                return;
            }
            const url = url_1.resolve(this.location.href, originalUrl);
            tasks.push({ url, callback: url => el.setAttribute(attrName, url) });
        };
        this.finder({ selector: '[src]', not: ['[src=""]', 'iframe'] }).map(el => {
            el.removeAttribute('srcset');
            process(el, 'src', 'vanillaClipperSrc');
        });
        this.finder({
            selector: '[href]',
            not: [
                '[href=""]',
                'a',
                'div',
                '[rel~=alternate]',
                '[rel~=canonical]',
                '[rel~=prev]',
                '[rel~=next]',
            ],
        }).map(el => {
            process(el, 'href', 'vanillaClipperHref');
        });
        await resource_1.batchCreateResources(tasks);
    }
    async optimizeStylesInShadowDOM() {
        await Promise.all(this.finder(element_1.buildVAttrSelector.shadowContent()).map(async (host) => {
            if (!host.dataset.vanillaClipperShadowContent) {
                return;
            }
            const shadow = host.attachShadow({ mode: 'open' });
            shadow.innerHTML = host.dataset.vanillaClipperShadowContent;
            const cssTexts = [];
            shadow.querySelectorAll('style').forEach(el => {
                cssTexts.push(el.innerHTML);
                el.remove();
            });
            const optimized = await Promise.all(cssTexts.map(text => css_1.optimizeCSS({ text, url: this.location.href }), shadow));
            this.appendStyleSheets(optimized, shadow);
            host.dataset.vanillaClipperShadowContent = shadow.innerHTML;
        }));
    }
    appendStyleSheets(cssTexts, shadowRoot) {
        const styleElements = cssTexts.map(text => {
            const el = this.document.createElement('style');
            el.dataset.vanillaClipperStyle = '';
            el.innerHTML = text;
            return el;
        });
        (shadowRoot ? shadowRoot : this.document.head).append(...styleElements);
    }
    appendScriptToHead() {
        const shadowHostSelector = element_1.buildVAttrSelector.shadowContent();
        const videoElementSelector = element_1.buildVAttrSelector.video();
        async function main() {
            if (document.readyState === 'loading') {
                document.addEventListener('DOMContentLoaded', onload);
            }
            else {
                onload();
            }
            function onload() {
                const videoElement = document.querySelector(videoElementSelector);
                if (videoElement) {
                    const playOrPause = () => videoElement.paused ? videoElement.play() : videoElement.pause();
                    videoElement.onclick = playOrPause;
                    document.body.onkeypress = () => {
                        if (window.event.keyCode === 32) {
                            ;
                            window.event.preventDefault();
                            playOrPause();
                        }
                    };
                }
                const shadowHosts = document.querySelectorAll(shadowHostSelector);
                shadowHosts.forEach(host => {
                    if (!host.dataset.vanillaClipperShadowContent) {
                        return;
                    }
                    const shadow = host.attachShadow({ mode: 'open' });
                    shadow.innerHTML = host.dataset.vanillaClipperShadowContent;
                });
            }
        }
        const script = this.document.createElement('script');
        script.dataset.vanillaClipperScript = '';
        script.innerHTML = `
            const shadowHostSelector = '${shadowHostSelector}'
            const videoElementSelector = '${videoElementSelector}'

            ${main.toString()}
            main()
        `;
        this.document.head.appendChild(script);
    }
}
exports.VJsdom = VJsdom;
//# sourceMappingURL=VJsdom.js.map