"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const appdata_path_1 = __importDefault(require("appdata-path"));
const crypto_1 = require("crypto");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
exports.dataURLPattern = /^data:([\w\/\+]+)(?:;.*)?,(.*)$/;
function dataURLToBuffer(dataURL) {
    const m = exports.dataURLPattern.exec(dataURL);
    if (!m) {
        return;
    }
    return { buffer: Buffer.from(m[2], 'base64'), mimetype: m[1] || undefined };
}
exports.dataURLToBuffer = dataURLToBuffer;
function extractExtensionFromURL(url) {
    const m = url.match(/['"]?[^?#]+\.(\w+)(?:['"]?$|\?|#)/);
    return m ? m[1] : null;
}
exports.extractExtensionFromURL = extractExtensionFromURL;
async function newFilePath(directory, basename, ext = 'html') {
    const fileList = await fs_extra_1.readdir(directory).catch(() => []);
    if (!fileList.includes(`${basename}.${ext}`)) {
        return path_1.resolve(directory, `${basename}.${ext}`);
    }
    const regex = new RegExp(`-(\\d+)\\.${ext}`);
    const numbers = fileList
        .map(name => name.startsWith(`${basename}-`) && name.match(regex))
        .filter((m) => !!m)
        .map(m => parseInt(m[1]));
    const number = Math.max(0, ...numbers) + 1;
    return path_1.resolve(directory, `${basename}-${number}.${ext}`);
}
exports.newFilePath = newFilePath;
exports.dataDirectoryPath = process.env.NODE_ENV === 'test' ? path_1.resolve('tmp', '__data__') : appdata_path_1.default('vanilla-clipper');
function dataPath(...pathSegments) {
    return path_1.resolve(exports.dataDirectoryPath, ...pathSegments);
}
exports.dataPath = dataPath;
exports.File = (path) => {
    return {
        path,
        async write(data) {
            return fs_extra_1.outputFile(path, data);
        },
        async readAsText() {
            return fs_extra_1.readFile(path, 'utf8');
        },
    };
};
exports.Directory = (path) => {
    return {
        path,
        childPath(name) {
            return path_1.resolve(path, name);
        },
        subdir(dirname) {
            return exports.Directory(this.childPath(dirname));
        },
        file(name) {
            return exports.File(this.childPath(name));
        },
        getChildren() {
            return fs_extra_1.readdirSync(path).map(this.childPath);
        },
        stat(name) {
            return fs_extra_1.statSync(this.childPath(name));
        },
        fileList() {
            return this.getChildren()
                .filter(path => this.stat(path).isFile())
                .map(exports.File);
        },
        dirList() {
            return this.getChildren()
                .filter(path => this.stat(path).isDirectory())
                .map(exports.Directory);
        },
    };
};
exports.dataDirectory = exports.Directory(exports.dataDirectoryPath);
exports.pagesDirectory = exports.dataDirectory.subdir('pages');
exports.pagesMainDirectory = exports.pagesDirectory.subdir('main');
exports.resourcesDirectory = exports.dataDirectory.subdir('resources');
exports.configFilePath = exports.dataDirectory.childPath('config.js');
exports.pouchdbConfigFilePath = exports.dataDirectory.childPath('pouchdb-config.json');
function getHash(buffer) {
    const hash = crypto_1.createHash('sha256');
    hash.update(buffer);
    return hash.digest('hex');
}
exports.getHash = getHash;
//# sourceMappingURL=file.js.map