"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const luxon_1 = require("luxon");
require("mime-types");
const mime_types_1 = require("mime-types");
const trysafe_1 = require("trysafe");
const ulid_1 = require("ulid");
const config_store_1 = require("../../server/config-store");
const utils_1 = require("../../server/utils");
const file_1 = require("../../server/utils/file");
const pouchdb_1 = require("../pouchdb");
exports.resources = pouchdb_1.VPouchdb('resources', {
    index: { fields: ['url', 'created_at'] },
});
function withRelativeURL(version) {
    return { version, url: `../../resources/${version.path}` };
}
async function createVersion({ url, buffer, hash, mimetype, }) {
    const outputDir = luxon_1.DateTime.local().toFormat('yyyyMMdd');
    const id = ulid_1.ulid();
    const ext = mimetype && mime_types_1.extension(mimetype);
    const filename = ext ? `${id}.${ext}` : id;
    const path = `${outputDir}/${filename}`;
    await file_1.resourcesDirectory.file(path).write(buffer);
    const doc = {
        url,
        created_at: utils_1.now(),
        hash,
        path,
    };
    await (await exports.resources).post(doc);
    return doc;
}
async function getVersions(url) {
    const result = await (await exports.resources).find({
        selector: {
            url,
            language: { $ne: 'query' },
        },
    });
    const versions = result.docs.sort((a, b) => {
        if (a.created_at > b.created_at) {
            return 1;
        }
        if (a.created_at < b.created_at) {
            return -1;
        }
        return 0;
    });
    return {
        recentVersion() {
            const last = versions[versions.length - 1];
            if (last && -1 < luxon_1.DateTime.fromISO(last.created_at).diffNow('hours').hours) {
                return last;
            }
        },
        findByHash(hash) {
            return versions.find(v => v.hash === hash);
        },
    };
}
async function batchCreateResources(tasks) {
    const urls = [...new Set(tasks.map(task => task.url))];
    await Promise.all(urls.map(async (url) => {
        const version = await storeResource(url);
        if (!version) {
            return;
        }
        tasks.filter(task => task.url === url).forEach(task => task.callback(version.url));
    })).catch(console.error);
}
exports.batchCreateResources = batchCreateResources;
async function storeResource(url) {
    const resource = await getVersions(url);
    const recentVersion = resource.recentVersion();
    if (recentVersion) {
        return withRelativeURL(recentVersion);
    }
    const result = await trysafe_1.TryAsync(async () => {
        const { headers: { 'content-type': mimetype, 'content-length': size }, } = await utils_1.got.head(url);
        if (size && parseInt(size) > config_store_1.config.resource.maxSize) {
            return;
        }
        const { body } = await utils_1.got.get(url, { encoding: null });
        if (!body.length || body.length > config_store_1.config.resource.maxSize) {
            return;
        }
        const response = { buffer: body, mimetype };
        const hash = file_1.getHash(response.buffer);
        const matchedVersion = resource.findByHash(hash);
        if (matchedVersion) {
            return withRelativeURL(matchedVersion);
        }
        return withRelativeURL(await createVersion({
            url,
            buffer: response.buffer,
            hash,
            mimetype: response.mimetype,
        }));
    });
    if (result.isLeft()) {
        utils_1.sig.warn(result.value);
        return;
    }
    return result.value;
}
exports.storeResource = storeResource;
//# sourceMappingURL=resource.js.map